@@ -1,15 +1,23 @@
 import math
+import warnings
 from random import randrange
 from time import time
+from typing import Any, Callable, List, Tuple
+
 from win10toast import ToastNotifier
-from typing import Any, List, Tuple, Callable
-import warnings
 
 
 class bigO:
-    """Big-O calculator
+    """
+    Big-O calculator
 
-    usage:
+    Methods
+    -------
+    test(function, str):
+        Returns the complexity and the execution time to sort arrays with function
+
+    Usage
+    -----
         from bigO import bigO
         bigO = bigO.bigO()
         bigO.test(someSort, "random")
@@ -19,7 +27,6 @@ class bigO:
         self.coef = 0.0
         self.rms = 0.0
         self.cplx = 0
-        self._ = 0
         self.O1 = 1
         self.ON = 2
         self.ON2 = 3
@@ -35,11 +42,11 @@ class bigO:
     def complexity2str(self, cplx: int) -> str:
         def switch(cplx):
             return {
-                self.ON: "O(N)",
-                self.ON2: "O(N^2)",
-                self.ON3: "O(N^3)",
-                self.OLogN: "O(lg(N)",
-                self.ONLogN: "O(Nlg(N))",
+                self.ON: "O(n)",
+                self.ON2: "O(n^2)",
+                self.ON3: "O(n^3)",
+                self.OLogN: "O(log(n)",
+                self.ONLogN: "O(nlog(n))",
                 self.O1: "O(1)",
             }.get(cplx, "f(n)")
 
@@ -107,13 +114,14 @@ class bigO:
         return result
 
     def estimate(self, n: List[int], times: List[float]):
+        assert len(n) == len(
+            times
+        ), f"ERROR: Length mismatch between N:{len(n)} and TIMES:{len(times)}."
+        assert len(n) >= 2, "ERROR: Need at least 2 runs."
+
         bestFit = bigO()
 
-        if len(n) != len(times):
-            return bestFit, "ERROR: Length mismatch between n and times."
-        if len(n) < 2:
-            return bestFit, "ERROR: Need at least 2 runs."
-
+        # assume that O1 is the best case
         bestFit = self.minimalLeastSq(n, times, self.fittingCurve(self.O1))
         bestFit.cplx = self.O1
 
@@ -123,87 +131,85 @@ class bigO:
                 bestFit = currentFit
                 bestFit.cplx = fit
 
-        return bestFit, None
+        return bestFit
 
-    def test(self, functionName: Callable, array: str) -> Tuple[str, str, List[Any]]:
-        """Big-O calculator
+    def genRandomArray(_, size):
+        array = []
+        for _ in range(size):
+            array.append(randrange(-size, size))
+        return array
+
+    def genSortedArray(_, size):
+        array = []
+        for value in range(size):
+            array.append(value)
+        return array
+
+    def genReversedArray(_, size):
+        array = []
+        for value in reversed(range(size)):
+            array.append(value)
+        return array
+
+    def genPartialArray(self, size):
+        array = self.genRandomArray(size)
+        sorted_array = self.genSortedArray(size)
+
+        array[size // 4 : size // 2] = sorted_array[size // 4 : size // 2]
+        return array
+
+    def genKsortedArray(self, size, k):
+        def reverseRange(array, a, b):
+            i = a
+            j = b - 1
+            while i < j:
+                array[i], array[j] = array[j], array[i]
+                i += 1
+                j -= 1
+
+            return array
+
+        assert size >= k, "K must be smaller than the size."
+        if size == 0:
+            return self.genSortedArray(size)
+        elif size == k:
+            return self.genReversedArray(size)
+
+        array = []
+        for value in range(-size // 2, size // 2):
+            array.append(value)
+
+        right = randrange(k)
+        while right > size - k - 1:  # Don't reverse again
+            right -= 1
+
+        if size != k:
+            k += 1
+
+        reverseRange(array, 0, k)
+        reverseRange(array, len(array) - right, len(array))
+
+        return array
+
+    def test(self, functionName: Callable, array: str) -> Tuple[str, float]:
+        """
+        ex) test(bubbleSort, "random")
 
         Args:
             functionName (Callable): a function to call |
-            array (string): "random", "sorted", "reversed", "partial", "Ksorted"
+            array (str): "random", "sorted", "reversed", "partial", "Ksorted"
 
         Returns:
-            complexity (string) : ex) O(N)
-            _ (string) : Error message else None
-            result (array) : Result of sorted array
+            complexity (str) : ex) O(n) |
+            time (float) : Time took to sort all 5 different arrays in second (max=100,000)
 
         """
-        # TODO : "all" to test all types of array
-        def genRandomArray(size):
-            array = []
-            for _ in range(size):
-                array.append(randrange(-size, size))
-            return array
-
-        def genSortedArray(size):
-            array = []
-            for value in range(size):
-                array.append(value)
-            return array
-
-        def genReversedArray(size):
-            array = []
-            for value in reversed(range(size)):
-                array.append(value)
-            return array
-
-        def genPartialArray(size):
-            array = genRandomArray(size)
-            sorted_array = genSortedArray(size)
-
-            array[size // 4 : size // 2] = sorted_array[size // 4 : size // 2]
-            return array
-
-        def genKsortedArray(size, k):
-            def reverseRange(array, a, b):
-                i = a
-                j = b - 1
-                while i < j:
-                    array[i], array[j] = array[j], array[i]
-                    i += 1
-                    j -= 1
-
-                return array
-
-            # ? can return random array
-            assert size > k, "K must be smaller than the size."
-            if size == 0:
-                return genSortedArray(size)
-            elif size == k:
-                return genReversedArray(size)
-
-            array = []
-            for value in range(-size // 2, size // 2):
-                array.append(value)
-
-            right = randrange(k)
-            while right > size - k - 1:  # Don't reverse again
-                right -= 1
-
-            if size != k:
-                k += 1
-
-            reverseRange(array, 0, k)
-            reverseRange(array, len(array) - right, len(array))
-
-            return array
-
+        # TODO : internal sorting algorithms, test all option
         bigOtest = bigO()
 
         sizes = [10, 100, 1000, 10000, 100000]
         maxIter = 5
         times = []
-        result = []
 
         toaster = ToastNotifier()
         print(f"Running {functionName.__name__}({array} array)...")
@@ -217,21 +223,22 @@ class bigO:
             timeTaken = 0.0
             nums = []
 
+            array = array.lower()
             if array == "random":
-                nums = genRandomArray(size)
+                nums = self.genRandomArray(size)
             elif array == "sorted":
-                nums = genSortedArray(size)
+                nums = self.genSortedArray(size)
             elif array == "partial":
-                nums = genPartialArray(size)
+                nums = self.genPartialArray(size)
             elif array == "reversed":
-                nums = genReversedArray(size)
-            elif array.lower() == "Ksorted":
-                nums = genKsortedArray(size, size // 10)
+                nums = self.genReversedArray(size)
+            elif array == "ksorted":
+                nums = self.genKsortedArray(size, size // 10)
             # elif array == "custom":
             #    nums = custom
             #    assert len(nums) != 0, "Please, pass the custom array you want.
             else:  # default = random array
-                nums = genRandomArray(size)
+                nums = self.genRandomArray(size)
 
             currentIter = 0
 
@@ -242,12 +249,7 @@ class bigO:
                 timeTaken += end - start
                 currentIter += 1
 
-                # ? if none pass or assert
-                if result == None:
-                    warnings.warn(
-                        "Please, return the array to see result.", UserWarning
-                    )
-                else:
+                if result != None:
                     assert result == sorted(
                         nums
                     ), "This function doesn't sort correctly."
@@ -255,12 +257,15 @@ class bigO:
             timeTaken /= maxIter
             times.append(timeTaken)
 
-        complexity, _ = bigOtest.estimate(sizes, times)
+        complexity = bigOtest.estimate(sizes, times)
+        estimatedTime = sum(times)
+
         print(f"Completed {functionName.__name__}({array} array): {complexity.str()}")
+        print(f"Time took: {estimatedTime:.5f}s")
         toaster.show_toast(
             "Big-O Caculator",
             f"Completed {functionName.__name__}({array} array): {complexity.str()}",
             duration=3,
         )
 
-        return complexity.str(), _, result
+        return complexity.str(), estimatedTime
